@@ -92,6 +92,11 @@ typedef void * gnutls_transport_ptr;
 
 #define HANDSHAKE_HEADER_SIZE 4
 
+/* defaults for verification functions
+ */
+#define DEFAULT_VERIFY_DEPTH 6
+#define DEFAULT_VERIFY_BITS 8200
+
 #include <gnutls_mem.h>
 #include <gnutls_ui.h>
 
@@ -102,6 +107,11 @@ typedef void * gnutls_transport_ptr;
 typedef unsigned char opaque;
 typedef struct { opaque pint[3]; } uint24;
 
+typedef struct {
+	opaque * data;
+	unsigned int size;
+} gnutls_datum;
+
 #include <gnutls_mpi.h>
 
 typedef enum ChangeCipherSpecType { GNUTLS_TYPE_CHANGE_CIPHER_SPEC=1 } ChangeCipherSpecType;
@@ -141,16 +151,6 @@ typedef enum HandshakeType { GNUTLS_HELLO_REQUEST, GNUTLS_CLIENT_HELLO, GNUTLS_S
 
 typedef HandshakeType gnutls_handshake_description;
 
-typedef struct {
-	opaque * data;
-	unsigned int size;
-} gnutls_datum;
-
-typedef struct {
-	const opaque * data;
-	unsigned int size;
-} gnutls_const_datum;
-
 #include <gnutls_buffer.h>
 
 /* This is the maximum number of algorithms (ciphers or macs etc).
@@ -215,6 +215,8 @@ typedef enum gnutls_x509_crt_fmt { GNUTLS_X509_FMT_DER,
 typedef enum gnutls_pk_algorithm { GNUTLS_PK_RSA = 1, GNUTLS_PK_DSA,
 	GNUTLS_PK_UNKNOWN = 0xff
 } gnutls_pk_algorithm;
+#define GNUTLS_PK_ANY (gnutls_pk_algorithm)-1
+#define GNUTLS_PK_NONE (gnutls_pk_algorithm)0
 
 typedef enum gnutls_sign_algorithm { GNUTLS_SIGN_RSA_SHA = 1, GNUTLS_SIGN_DSA_SHA,
 	GNUTLS_SIGN_RSA_MD5, GNUTLS_SIGN_RSA_MD2,
