@@ -17,7 +17,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
  */
-
+ 
 
 /*****************************************************/
 /* File: decoding.c                                  */
@@ -64,7 +64,7 @@ _asn1_get_length_der(const unsigned char *der,int  *len)
       ans=0;
       while(punt<=k) ans=ans*256+der[punt++];
     }
-    else{  /* indefinite lenght method */
+    else{  /* indefinite length method */
       ans=-1;
     }
 
@@ -122,17 +122,22 @@ _asn1_get_octet_der(const unsigned char *der,int *der_len,unsigned char *str,int
 
 
 
-
-void
-_asn1_get_time_der(const unsigned char *der,int *der_len,unsigned char *str)
+/* Returns ASN1_SUCCESS on success or an error code on error.
+ */
+int
+_asn1_get_time_der(const unsigned char *der,int *der_len,unsigned char *str,int str_size)
 {
   int len_len,str_len;
 
-  if(str==NULL) return;
+  if(str==NULL) return ASN1_DER_ERROR;
   str_len=_asn1_get_length_der(der,&len_len);
+  if (str_len < 0 || str_size < str_len) 
+     return ASN1_DER_ERROR;
   memcpy(str,der+len_len,str_len);
   str[str_len]=0;
   *der_len=str_len+len_len;
+  
+  return ASN1_SUCCESS;
 }
 
 
@@ -406,33 +411,36 @@ _asn1_get_octet_string(const unsigned char* der,node_asn *node,int* len)
     }
 
     /* copy */
-    _asn1_length_der(tot_len,NULL,&len2);
-    temp=(unsigned char *)_asn1_alloca(len2+tot_len);
-    if (temp==NULL){
+    if(node){
+      _asn1_length_der(tot_len,NULL,&len2);
+      temp=(unsigned char *)_asn1_alloca(len2+tot_len);
+      if (temp==NULL){
 	return ASN1_MEM_ALLOC_ERROR;
-    }
-    
-    _asn1_length_der(tot_len,temp,&len2);
-    tot_len+=len2;
-    temp2=temp+len2;
-    len2=_asn1_get_length_der(der,&len3);
-    counter2=len3+1;
+      }
+      
+      _asn1_length_der(tot_len,temp,&len2);
+      tot_len+=len2;
+      temp2=temp+len2;
+      len2=_asn1_get_length_der(der,&len3);
+      counter2=len3+1;
 
-    if(indefinite==-1) counter_end=counter-2;
-    else counter_end=counter;
-
-    while(counter2<counter_end){
-      len2=_asn1_get_length_der(der+counter2,&len3);	
-      memcpy(temp2,der+counter2+len3,len2);
-      temp2+=len2;
-      counter2+=len2+len3+1;
+      if(indefinite==-1) counter_end=counter-2;
+      else counter_end=counter;
+      
+      while(counter2<counter_end){
+	len2=_asn1_get_length_der(der+counter2,&len3);	
+	memcpy(temp2,der+counter2+len3,len2);
+	temp2+=len2;
+	counter2+=len2+len3+1;
+      }
+      _asn1_set_value(node,temp,tot_len);
+      _asn1_afree(temp);
     }
-    _asn1_set_value(node,temp,tot_len);
-    _asn1_afree(temp);
   }
   else{  /* NOT STRUCTURED */
     len2=_asn1_get_length_der(der,&len3);
-    _asn1_set_value(node,der,len3+len2);
+    if(node)
+      _asn1_set_value(node,der,len3+len2);
     counter=len3+len2;
   }   
 
@@ -482,13 +490,13 @@ _asn1_get_indefinite_length_string(const unsigned char* der,int* len)
 /**
   * asn1_der_decoding - Fill the structure *ELEMENT with values of a DER encoding string.
   * @element: pointer to an ASN1 structure
-  * @der: vector that contains the DER encoding. 
+  * @ider: vector that contains the DER encoding. 
   * @len: number of bytes of *der: der[0]..der[len-1]
-  * Description:
+  * @errorDescription: null-terminated string contains details when an error occurred.
   *
   * Fill the structure *ELEMENT with values of a DER encoding string. The sructure must just be
   * created with function 'create_stucture'.
-  * If an error accurs during de decoding procedure, the *ELEMENT is deleted
+  * If an error occurs during the decoding procedure, the *ELEMENT is deleted
   * and set equal to ASN1_TYPE_EMPTY.
   *
   * Returns:
@@ -501,15 +509,16 @@ _asn1_get_indefinite_length_string(const unsigned char* der,int* len)
   **/
 
 asn1_retCode
-asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
+asn1_der_decoding(ASN1_TYPE *element,const void *ider,int len,
 		  char *errorDescription)
 {
   node_asn *node,*p,*p2,*p3;
   char temp[128];
-  int counter,len2,len3,len4,move,ris;
+  int counter,len2,len3,len4,move,ris,tlen;
   unsigned char class,*temp2;
   unsigned int tag;
-  int indefinite;
+  int indefinite, result;
+  const unsigned char* der = ider;
 
   node=*element;
 
@@ -525,7 +534,6 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
   p=node;
   while(1){
     ris=ASN1_SUCCESS;
-
     if(move!=UP){
       if(p->type&CONST_SET){
 	p2=_asn1_find_up(p);
@@ -574,6 +582,23 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
 	}
       }
 
+      if((p->type&CONST_OPTION) || (p->type&CONST_DEFAULT)){
+	p2=_asn1_find_up(p);
+	len2=strtol(p2->value,NULL,10);
+	if(counter==len2){
+	  if(p->right){
+	    p2=p->right;
+	    move=RIGHT;
+	  }
+	  else move=UP;
+
+	  if(p->type&CONST_OPTION) asn1_delete_structure(&p);
+
+	  p=p2;
+	  continue;
+	}
+      }
+
       if(type_field(p->type)==TYPE_CHOICE){
 	while(p->down){
 	  if(counter<len)
@@ -597,7 +622,7 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
 	  }
 	}
 
- 	if(p->down==NULL){
+        if(p->down==NULL){
 	  if(!(p->type&CONST_OPTION)){
 	    asn1_delete_structure(element);
 	    return ASN1_DER_ERROR;
@@ -610,9 +635,9 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
       if((p->type&CONST_OPTION) || (p->type&CONST_DEFAULT)){
 	p2=_asn1_find_up(p);
 	len2=strtol(p2->value,NULL,10);
-	if((len2!=-1) && (counter>=len2)) ris=ASN1_TAG_ERROR;
+	if((len2!=-1) && (counter>len2)) ris=ASN1_TAG_ERROR;
       }
-  
+     
       if(ris==ASN1_SUCCESS) ris=_asn1_extract_tag_der(p,der+counter,&len2);
       if(ris!=ASN1_SUCCESS){
 	if(p->type&CONST_OPTION){
@@ -661,13 +686,21 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
 	break;
       case TYPE_OBJECT_ID:
 	_asn1_get_objectid_der(der+counter,&len2, temp, sizeof(temp));
-	_asn1_set_value(p,temp,strlen(temp)+1);
+	tlen = strlen(temp);
+	if( tlen > 0)
+	   _asn1_set_value(p,temp,tlen+1);
 	counter+=len2;
 	move=RIGHT;
       break;
       case TYPE_TIME:
-	_asn1_get_time_der(der+counter,&len2,temp);
-	_asn1_set_value(p,temp,strlen(temp)+1);
+	result = _asn1_get_time_der(der+counter,&len2,temp,sizeof(temp)-1);
+	if (result != ASN1_SUCCESS) {
+		asn1_delete_structure(element);
+	        return result;
+	}
+	tlen = strlen(temp);
+	if (tlen > 0)
+	   _asn1_set_value(p,temp,tlen+1);
 	counter+=len2;
 	move=RIGHT;
 	break;
@@ -690,7 +723,7 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
 	counter+=len3+len2;
 	move=RIGHT;
 	break;
-      case TYPE_SEQUENCE:  case TYPE_SET:;
+      case TYPE_SEQUENCE:  case TYPE_SET:
 	if(move==UP){
 	  len2=strtol(p->value,NULL,10);
 	  _asn1_set_value(p,NULL,0);
@@ -714,7 +747,9 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
 	  counter+=len2;
 	  if(len3>0){
 	    _asn1_ltostr(counter+len3,temp);
-	    _asn1_set_value(p,temp,strlen(temp)+1);
+	    tlen = strlen(temp);
+	    if (tlen > 0)
+	       _asn1_set_value(p,temp,tlen+1);
 	    move=DOWN; 
 	  }
 	  else if(len3==0){
@@ -772,7 +807,10 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
 	  if(len3){
 	    if(len3>0){ /* definite length method */
 	    _asn1_ltostr(counter+len3,temp);
-	    _asn1_set_value(p,temp,strlen(temp)+1);
+	    tlen = strlen(temp);
+	    
+	    if (tlen > 0)
+	       _asn1_set_value(p,temp,tlen+1);
 	    }
 	    else { /* indefinite length method */
 	      _asn1_set_value(p,"-1",3);	      
@@ -786,12 +824,6 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
 	move=RIGHT;
 	break;
       case TYPE_ANY:
-	/* Check indefinite lenth method in a EXPLICIT TAG */
-	if((p->type&CONST_TAG) && (der[counter-1]==0x80))
-	  indefinite=1;
-	else
-	  indefinite=0;
-
 	tag=_asn1_get_tag_der(der+counter,&class,&len2);
 	len4=_asn1_get_length_der(der+counter+len2,&len3);
 	
@@ -810,6 +842,12 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
 	  counter+=len2+len3;
 	}
 	else{ /* indefinite length */
+	  /* Check indefinite lenth method in an EXPLICIT TAG */
+	  if((p->type&CONST_TAG) && (der[counter-1]==0x80))
+	    indefinite=1;
+	  else
+	    indefinite=0;
+
 	  len2=len-counter;
 	  ris=_asn1_get_indefinite_length_string(der+counter,&len2);
 	  if(ris != ASN1_SUCCESS){
@@ -827,17 +865,17 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
 	  _asn1_set_value(p,temp2,len4);
 	  _asn1_afree(temp2);
 	  counter+=len2;
-	}
 
-	/* Check if a couple of 0x00 are present due to an EXPLICIT TAG with
-	   a indefinite length method. */
-	if(indefinite){
-	  if(!der[counter] && !der[counter+1]){ 
-	    counter+=2;
-	  }
-	  else{
-	    asn1_delete_structure(element);
-	    return ASN1_DER_ERROR;
+	  /* Check if a couple of 0x00 are present due to an EXPLICIT TAG with
+	     an indefinite length method. */
+	  if(indefinite){
+	    if(!der[counter] && !der[counter+1]){ 
+	      counter+=2;
+	    }
+	    else{
+	      asn1_delete_structure(element);
+	      return ASN1_DER_ERROR;
+	    }
 	  }
 	}
 	move=RIGHT;
@@ -881,16 +919,16 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
   * asn1_der_decoding_element - Fill the element named ELEMENTNAME of the structure STRUCTURE with values of a DER encoding string.
   * @structure: pointer to an ASN1 structure
   * @elementName: name of the element to fill
-  * @der: vector that contains the DER encoding of the whole structure. 
+  * @ider: vector that contains the DER encoding of the whole structure. 
   * @len: number of bytes of *der: der[0]..der[len-1]
-  * @errorDescription: null-terminated string contains details when an arror accured.
+  * @errorDescription: null-terminated string contains details when an error occurred.
   * 
   * Description:
   *
   * Fill the element named ELEMENTNAME with values of a DER encoding string. 
   * The sructure must just be created with function 'create_stucture'.
   * The DER vector must contain the encoding string of the whole STRUCTURE.
-  * If an error accurs during the decoding procedure, the *STRUCTURE is deleted
+  * If an error occurs during the decoding procedure, the *STRUCTURE is deleted
   * and set equal to ASN1_TYPE_EMPTY.
   *
   * Returns:
@@ -904,15 +942,16 @@ asn1_der_decoding(ASN1_TYPE *element,const unsigned char *der,int len,
   **/
 asn1_retCode
 asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
-			  const unsigned char *der,int len,char *errorDescription)
+			  const void *ider,int len,char *errorDescription)
 {
   node_asn *node,*p,*p2,*p3,*nodeFound=ASN1_TYPE_EMPTY;
   char temp[128],currentName[MAX_NAME_SIZE*10],*dot_p,*char_p;
   int nameLen=MAX_NAME_SIZE*10-1,state;
-  int counter,len2,len3,len4,move,ris;
+  int counter,len2,len3,len4,move,ris, tlen;
   unsigned char class,*temp2;
   unsigned int tag;
-  int indefinite;
+  int indefinite, result;
+  const unsigned char* der = ider;
 
   node=*structure;
 
@@ -928,7 +967,7 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
     return ASN1_GENERIC_ERROR;
   }
 
-  if((*structure)->name){  /* Has *structure a name? */
+  if((*structure)->name){  /* Has *structure got a name? */
     nameLen-=strlen((*structure)->name);
     if(nameLen>0) strcpy(currentName,(*structure)->name);
     else{
@@ -1002,6 +1041,23 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
 	}
       }
 
+      if((p->type&CONST_OPTION) || (p->type&CONST_DEFAULT)){
+	p2=_asn1_find_up(p);
+	len2=strtol(p2->value,NULL,10);
+	if(counter==len2){
+	  if(p->right){
+	    p2=p->right;
+	    move=RIGHT;
+	  }
+	  else move=UP;
+
+	  if(p->type&CONST_OPTION) asn1_delete_structure(&p);
+
+	  p=p2;
+	  continue;
+	}
+      }
+
       if(type_field(p->type)==TYPE_CHOICE){
 	while(p->down){
 	  if(counter<len)
@@ -1038,7 +1094,7 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
       if((p->type&CONST_OPTION) || (p->type&CONST_DEFAULT)){
 	p2=_asn1_find_up(p);
       	len2=strtol(p2->value,NULL,10);
-	if(counter>=len2) ris=ASN1_TAG_ERROR;
+	if(counter>len2) ris=ASN1_TAG_ERROR;
       }
 
       if(ris==ASN1_SUCCESS) ris=_asn1_extract_tag_der(p,der+counter,&len2);
@@ -1106,7 +1162,10 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
       case TYPE_OBJECT_ID:
 	if(state==FOUND){
 	  _asn1_get_objectid_der(der+counter,&len2, temp, sizeof(temp));
-	  _asn1_set_value(p,temp,strlen(temp)+1);
+	  tlen = strlen(temp);
+	  
+	  if (tlen > 0)
+	     _asn1_set_value(p,temp,tlen+1);
 	  
 	  if(p==nodeFound) state=EXIT;
 	}
@@ -1120,8 +1179,15 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
       break;
       case TYPE_TIME:
 	if(state==FOUND){
-	  _asn1_get_time_der(der+counter,&len2,temp);
-	  _asn1_set_value(p,temp,strlen(temp)+1);
+	  result = _asn1_get_time_der(der+counter,&len2,temp,sizeof(temp)-1);
+	  if (result != ASN1_SUCCESS) {
+		asn1_delete_structure(structure);
+	        return result;
+	  }
+
+	  tlen = strlen(temp);
+	  if (tlen > 0)
+  	     _asn1_set_value(p,temp,tlen+1);
 	  
 	  if(p==nodeFound) state=EXIT;
 	}
@@ -1134,13 +1200,16 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
 	move=RIGHT;
 	break;
       case TYPE_OCTET_STRING:
-	len2=_asn1_get_length_der(der+counter,&len3);
+	len3=len-counter;
 	if(state==FOUND){
-	  _asn1_set_value(p,der+counter,len3+len2);
-	 
+	  ris=_asn1_get_octet_string(der+counter,p,&len3);
 	  if(p==nodeFound) state=EXIT;
 	}
-	counter+=len3+len2;
+	else
+	  ris=_asn1_get_octet_string(der+counter,NULL,&len3);
+
+	if(ris != ASN1_SUCCESS) return ris; 
+	counter+=len3;
 	move=RIGHT;
 	break;
       case TYPE_GENERALSTRING:
@@ -1194,7 +1263,10 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
 	    counter+=len2;
 	    if(len3>0){
 	      _asn1_ltostr(counter+len3,temp);
-	      _asn1_set_value(p,temp,strlen(temp)+1);
+	      tlen = strlen(temp);
+	      
+	      if(tlen > 0)
+	          _asn1_set_value(p,temp,tlen+1);
 	      move=DOWN;
 	    }
 	    else if(len3==0){
@@ -1246,7 +1318,10 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
 	    counter+=len2;
 	    if(len3){
 	      _asn1_ltostr(counter+len3,temp);
-	      _asn1_set_value(p,temp,strlen(temp)+1);
+	      tlen = strlen(temp);
+	      
+	      if (tlen > 0)
+	          _asn1_set_value(p,temp,tlen+1);
 	      p2=p->down;
 	      while((type_field(p2->type)==TYPE_TAG) || (type_field(p2->type)==TYPE_SIZE)) p2=p2->right;
 	      if(p2->right==NULL) _asn1_append_sequence_set(p);
@@ -1258,43 +1333,73 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
 	
 	break;
       case TYPE_ANY:
-	/* Check indefinite lenth method in a EXPLICIT TAG */
-	if((p->type&CONST_TAG) && (der[counter-1]==0x80))
-	  indefinite=1;
-	else
-	  indefinite=0;
-
 	tag=_asn1_get_tag_der(der+counter,&class,&len2);
-	len2+=_asn1_get_length_der(der+counter+len2,&len3);
-	if(state==FOUND){
-	  _asn1_length_der(len2+len3,NULL,&len4);
-	  temp2=(unsigned char *)_asn1_alloca(len2+len3+len4);
-	  if (temp2==NULL){
-	    asn1_delete_structure(structure);
-	    return ASN1_MEM_ALLOC_ERROR;
+	len4=_asn1_get_length_der(der+counter+len2,&len3);
+	
+	if(len4 != -1){
+	  len2+=len4;
+	  if(state==FOUND){
+	    _asn1_length_der(len2+len3,NULL,&len4);
+	    temp2=(unsigned char *)_asn1_alloca(len2+len3+len4);
+	    if (temp2==NULL){
+	      asn1_delete_structure(structure);
+	      return ASN1_MEM_ALLOC_ERROR;
+	    }
+	    
+	    _asn1_octet_der(der+counter,len2+len3,temp2,&len4);
+	    _asn1_set_value(p,temp2,len4);
+	    _asn1_afree(temp2);
+
+	    if(p==nodeFound) state=EXIT;
 	  }
+	  counter+=len2+len3;
+	}
+	else{ /* indefinite length */
+	  /* Check indefinite lenth method in an EXPLICIT TAG */
+	  if((p->type&CONST_TAG) && (der[counter-1]==0x80))
+	    indefinite=1;
+	  else
+	    indefinite=0;
+
+	  len2=len-counter;
+	  ris=_asn1_get_indefinite_length_string(der+counter,&len2);
+	  if(ris != ASN1_SUCCESS){
+	    asn1_delete_structure(structure);
+	    return ris;
+	  }
+	  
+	  if(state==FOUND){
+	    _asn1_length_der(len2,NULL,&len4);
+	    temp2=(unsigned char *)_asn1_alloca(len2+len4);
+	    if (temp2==NULL){
+	      asn1_delete_structure(structure);
+	      return ASN1_MEM_ALLOC_ERROR;
+	    }
         
-	_asn1_octet_der(der+counter,len2+len3,temp2,&len4);
-	_asn1_set_value(p,temp2,len4);
-	_asn1_afree(temp2);
+	    _asn1_octet_der(der+counter,len2,temp2,&len4);
+	    _asn1_set_value(p,temp2,len4);
+	    _asn1_afree(temp2);
 
-	/* Check if a couple of 0x00 are present due to an EXPLICIT TAG with
-           a indefinite length method. */
-	if(indefinite){
-	  if(!der[counter] && !der[counter+1]) 
-	    counter+=2;
-	  else{
-	    asn1_delete_structure(structure);
-	    return ASN1_DER_ERROR;
+	    if(p==nodeFound) state=EXIT;
+	  }
+
+	  counter+=len2;
+
+	  /* Check if a couple of 0x00 are present due to an EXPLICIT TAG with
+	     an indefinite length method. */
+	  if(indefinite){
+	    if(!der[counter] && !der[counter+1]){ 
+	      counter+=2;
+	    }
+	    else{
+	      asn1_delete_structure(structure);
+	      return ASN1_DER_ERROR;
+	    }
 	  }
 	}
-       
-	if(p==nodeFound) state=EXIT;
-	}
-
-	counter+=len2+len3;
 	move=RIGHT;
 	break;
+
       default:
 	move=(move==UP)?RIGHT:DOWN;
 	break;
@@ -1404,7 +1509,7 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
 /**
   * asn1_der_decoding_startEnd - Find the start and end point of an element in a DER encoding string.
   * @element: pointer to an ASN1 element
-  * @der: vector that contains the DER encoding. 
+  * @ider: vector that contains the DER encoding. 
   * @len: number of bytes of *der: der[0]..der[len-1]
   * @name_element: an element of NAME structure.
   * @start: the position of the first byte of NAME_ELEMENT decoding (der[*start]) 
@@ -1427,14 +1532,15 @@ asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
   *
   **/
 asn1_retCode
-asn1_der_decoding_startEnd(ASN1_TYPE element,const unsigned char *der,int len,
+asn1_der_decoding_startEnd(ASN1_TYPE element,const void *ider,int len,
 			   const char *name_element,int *start, int *end)
 {
   node_asn *node,*node_to_find,*p,*p2,*p3;
-  int counter,len2,len3,move,ris;
+  int counter,len2,len3,len4,move,ris;
   unsigned char class;
   unsigned int tag;
   int indefinite;
+  const unsigned char* der = ider;
 
   node=element;
 
@@ -1548,8 +1654,10 @@ asn1_der_decoding_startEnd(ASN1_TYPE element,const unsigned char *der,int len,
 	move=RIGHT;
 	break;
       case TYPE_OCTET_STRING:
-	len2=_asn1_get_length_der(der+counter,&len3);
-	counter+=len3+len2;
+	len3=len-counter;
+	ris=_asn1_get_octet_string(der+counter,NULL,&len3);
+	if(ris != ASN1_SUCCESS) return ris; 
+	counter+=len3;
 	move=RIGHT;
 	break;
       case TYPE_GENERALSTRING:
@@ -1595,24 +1703,35 @@ asn1_der_decoding_startEnd(ASN1_TYPE element,const unsigned char *der,int len,
 	move=RIGHT;
 	break;
       case TYPE_ANY:
-	/* Check indefinite lenth method in a EXPLICIT TAG */
-	if((p->type&CONST_TAG) && (der[counter-1]==0x80))
-	  indefinite=1;
-	else
-	  indefinite=0;
-
 	tag=_asn1_get_tag_der(der+counter,&class,&len2);
-	len2+=_asn1_get_length_der(der+counter+len2,&len3);
-	counter+=len3+len2;
-	/* Check if a couple of 0x00 are present due to an EXPLICIT TAG with
-           a indefinite length method. */
-	if(indefinite){
-	  if(!der[counter] && !der[counter+1]) 
-	    counter+=2;
-	  else
-	    return ASN1_DER_ERROR;
+	len4=_asn1_get_length_der(der+counter+len2,&len3);
+	
+	if(len4 != -1){
+	  counter+=len2+len4+len3;
 	}
- 	move=RIGHT;
+	else{ /* indefinite length */
+	  /* Check indefinite lenth method in an EXPLICIT TAG */
+	  if((p->type&CONST_TAG) && (der[counter-1]==0x80))
+	    indefinite=1;
+	  else
+	    indefinite=0;
+	  
+	  len2=len-counter;
+	  ris=_asn1_get_indefinite_length_string(der+counter,&len2);
+	  if(ris != ASN1_SUCCESS)
+	    return ris;
+	  counter+=len2;
+
+	  /* Check if a couple of 0x00 are present due to an EXPLICIT TAG with
+	     an indefinite length method. */
+	  if(indefinite){
+	    if(!der[counter] && !der[counter+1])
+	      counter+=2;
+	    else
+	      return ASN1_DER_ERROR;
+	  }
+	}
+	move=RIGHT;
 	break;
       default:
 	move=(move==UP)?RIGHT:DOWN;
@@ -1643,8 +1762,7 @@ asn1_der_decoding_startEnd(ASN1_TYPE element,const unsigned char *der,int len,
 
 
 /**
-  * asn1_expand_any_defined_by - Expand every "ANY DEFINED BY" fields of 
-  *    structure *ELEMENT with the corresponding type.
+  * asn1_expand_any_defined_by - Expand "ANY DEFINED BY" fields in structure.
   * @definitions: ASN1 definitions
   * @element: pointer to an ASN1 structure
   * Description:
@@ -1839,8 +1957,7 @@ asn1_expand_any_defined_by(ASN1_TYPE definitions,ASN1_TYPE *element)
 
 
 /**
-  * asn1_expand_octet_string - Expand an "OCTET STRING" fields of 
-  *    structure *ELEMENT with the corresponding type.
+  * asn1_expand_octet_string - Expand "OCTET STRING" fields in structure.
   * @definitions: ASN1 definitions
   * @element: pointer to an ASN1 structure
   * @octetName: name of the OCTECT STRING field to expand.
@@ -1971,9 +2088,3 @@ asn1_expand_octet_string(ASN1_TYPE definitions,ASN1_TYPE *element,
   return retCode;
 }
 
-
-
-
-
-
-
