@@ -1,6 +1,6 @@
 /*
+ *  Copyright (C) 2004, 2005  Free Software Foundation
  *  Copyright (C) 2003 Nikos Mavroyanopoulos
- *  Copyright (C) 2004 Free Software Foundation
  *
  *  This file is part of GNUTLS.
  *
@@ -315,7 +315,7 @@ int _gnutls_x509_parse_dn(ASN1_TYPE asn1_struct,
 		memcpy(buf, out_str.data, out_str.length);
 		buf[out_str.length] = 0;
 	}
-	*sizeof_buf = out_str.length;
+	*sizeof_buf = out_str.length + 1;
 
 	result = 0;
 
@@ -466,6 +466,8 @@ int _gnutls_x509_parse_dn_oid(ASN1_TYPE asn1_struct,
 
 				if (result != ASN1_SUCCESS) {
 					gnutls_assert();
+					if (result==ASN1_MEM_ERROR)
+					    *sizeof_buf = len;
 					result = _gnutls_asn2err(result);
 					goto cleanup;
 				}
